%
% Copyright (C) Dale L. Ang
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Main CLI interface for the SOMSpec suite. Can be run from command line with:
%
% matlab -nodesktop -nosplash -r SOMSpec
%

function SOMSpec()
clearvars; % Clear all variables
dbstop if error; % Turn on debugging
close all; %
tic;

%% What do you want to perform?
train = 1; % Train a SOM. 1 = Yes, 0 = No.
test = 0; % Run a prediction. 1 = Yes, 0 = No.
validate = 0; % Run the validation suite. 1 = Yes, 0 = No.
spectraIsIR = 1; % If 1 (=Yes) bypasses scaling as the input is already normalised.
%normalise = 0; % Normalise spectra as it is imported? Not yet implemented.

%% File locations
workingPath = 'TEST/'; % Location of subdirectory containing working files (format "this/dir" - no leading or trailing /)
validationPath = 'notused'; % Subdirectory for validation results to be output - not used in validation mode
refSetFileVa = 'SP175_full_240-175_5P'; % TRAINING SET filename - eg. IRset for IR, CDset48 for CD, etc
refSetFileExt = '.txt'; % file extension of training set file - .txt or .csv
testSetFileVa = 'SP175_test'; % TEST SET filename - eg. IRtestset2 - don't include .txt
testSetFileExt = '.txt'; % file extension of test set - .txt or .csv
parametersFile = '';

%% Training Parameters
mapLength = 30; % Values between 30 and 40 are realistic for training sets containing ~50 spectra.
numIterations = 30000; % Values between 20000 and 100000 are realistic
numStructures = 5; % Defines number of properties the training set contains (=number of rows at end of input file) IRset = 5, CDset48 = 6.
L0 = 0.1; % Initial learning rate, typically 0.1 or less. Exponentially decays.
numBMUS = 3;  % Typically 5, but can be varied.
wavelengthLong = 175; % Used for calculating appropriate interpolation and plotting. MUST be the larger number.
wavelengthShort = 240;
modelSizeInt = 300; % Number of points in interpolated spectra.
waveUnits = 'cm-1'; % 'cm-1' or 'nm' currently - the units for plotting and some internal calculations

%% Visualisation options
realtimePlots = 0; % Display real time plots? 0 for no, 1 for yes. Warning: heavily impacts performance.
displayFinalPlots = 0; % Display the final trained plots.
displayFinalTestPlots = 0; % Display the SS content plots.
displayTestPlots = 0; % Display the predicted spectra and SOM plots as they are generated? The plots are still output to file as PDFs.
displayResampledSpectra = 0; % Display the input and resampled 
displayProgressBar = 0; % Switch to stop progress bar

%% Don't change anything below here
numLines = 0; % Initialise value.
numCols = 0; % Initialise value.
currentSpectrum = 0; % Initialise.
trainingSetSize = 0; % Initial value. Actual value calculated in SSTRAIN.m
numPointsPerStep = 4; % Interpolation sample points per step, eg. 4 = 0.25 nm

% Check for wavelength numbers in correct order
if wavelengthLong < wavelengthShort
   wavelengthLongtemp = wavelengthLong;
   wavelengthLong = wavelengthShort;
   wavelengthShort = wavelengthLongtemp;   
end

if validate == 1 % rename to the refSet name if validation running
   testSetFileVa = refSetFileVa;
end
savePath = strcat(pwd,filesep,workingPath,filesep); % Output location
mapLengthStr = num2str(mapLength);
numIterationsStr = num2str(numIterations);
SOMprefix = strcat('SOM','-',mapLengthStr,'-',numIterationsStr); % prefix for naming of trained SOM
savePathSOM = ''; %strcat(savePath,SOMprefix); % may be better to handle in the individual functions
savePathTest = strcat(pwd,filesep,workingPath,filesep,testSetFileVa,filesep); %???

radius0 = mapLength/2; % Radius initialised at half map length.
t1 = numIterations/(log(radius0)); % Calculate the time constant.
%radius0 = sqrt(2*mapLength^2); % Radius is max distance on map?
refSize = 0; % Initialise only
refPath = strcat(pwd,filesep,workingPath,filesep);
refPathSOM = strcat(pwd,filesep,workingPath,filesep,'SOM-',num2str(mapLength),'-',num2str(numIterations));
modelSize = 0; % Initialise only
testSize = 0; % Initialise only
%valmapLength = [10 20 30 40 50]; % Map lengths to use for validation
%valIterations = [1000 5000 10000 20000 30000 40000 50000]; % Iteration steps for validation

%% Scaling factor for concentration - predefined options are presented below
%concError = linspace(0.99,1.01,10); % Very small adjustments
concError = linspace(0.1,10,100); % 0.1 to 10 in 0.1 steps

% For validation only
V = ''; % Initialise as nothing - only used for VAL
W = ''; % Initialise as nothing - only used for VAL

% Check for canelled run
isCancelled = 0;

%% Pass the values to the functions as structs
% Structure array containing the filename information.
Filenames = struct(     'savePath',          savePath, ...
                        'savePathTest',      savePathTest,...
                        'workingPath',       workingPath, ...
                        'refSetFileVa',      refSetFileVa, ...
                        'refSetFileExt',     refSetFileExt, ...
                        'testSetFileVa',     testSetFileVa, ...
                        'testSetFileExt',    testSetFileExt, ...
                        'SOMprefix',         SOMprefix, ...
                        'savePathSOM',       savePathSOM, ...
                        'refPath',           refPath, ...
                        'refPathSOM',        refPathSOM, ...
                        'validationPath',    validationPath, ...
                        'parametersFile',    parametersFile);

% Structure array containing the parameter information.
Parameters = struct(    'mapLength',         mapLength, ...
                        'numIterations',     numIterations,...
                        'numStructures',     numStructures, ...
                        'numLines',          numLines, ...
                        'numCols',           numCols, ...
                        'realtimePlots',     realtimePlots, ...
                        'displayFinalPlots', displayFinalPlots, ...
                        'displayFinalTestPlots', displayFinalTestPlots, ...
                        'displayTestPlots',  displayTestPlots, ...
                        'displayResampledSpectra', displayResampledSpectra, ...
                        'displayProgressBar', displayProgressBar, ...
                        'L0',                L0, ...
                        'numBMUS',           numBMUS, ...
                        'modelSize',         modelSize, ...
                        'modelSizeInt',      modelSizeInt, ...
                        'testSize',          testSize, ...
                        't1',                t1, ...
                        'currentSpectrum',   currentSpectrum, ...
                        'trainingSetSize',   trainingSetSize, ...
                        'radius0',           radius0, ...
                        'refSize',           refSize, ...
                        'concError',         concError, ...
                        'validate',          validate, ...
                        'spectraIsIR',       spectraIsIR, ...
                        'V',                 V, ...
                        'W',                 W, ...
                        'isCancelled',       isCancelled, ...
                        'isGUI',             0, ...
                        'SOMplotStyle',      2, ...
                        'warning',           0, ...
                        'wavelengthLong',    wavelengthLong, ...
                        'wavelengthShort',   wavelengthShort, ...
                        'numPointsPerStep',  numPointsPerStep, ...
                        'nodeXtoPlot',       1, ...
                        'nodeYtoPlot',       1, ...
                        'waveUnits',         waveUnits);

if train == 1 
   [Filenames,Parameters] = SSTRAIN(Filenames,Parameters); % Run the SOM training code
end

if test == 1
   [Filenames,Parameters] = SSTEST(Filenames,Parameters); % Run the prediction code
end

if validate == 1
   SSVALIDATE(Filenames,Parameters); % Run the validation code
end

tocString = num2str(toc);
if validate == 1
   fprintf('%s %s %s\n','SOMSpec validation completed in',tocString,'sec.');
else
   fprintf('%s %s %s\n','SOMSpec execution completed in',tocString,'sec.');
end
fclose('all');
clear all;
end
