%
% Copyright (C) Dale L. Ang
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Produces correctly formatted (master) training set from PCDDB .pcd files.
% (SP175 and SMP180 for now - downloaded from PCDDB)
% Input is a directory of CD spectra. Data MUST be formatted from long to short
% wavelength, ie. 240 - 190 nm and span the required training sets.
%

function SSPCDDB()
%% Define input variables here.
clear all;
fclose('all');
inputPath = 'refsets/CD_PCDDB/SP175/PCD'; % Directory containing the .pcd files
SSWLstart = '240.0'; % Data set import start
SSWLend = '175.0'; % Data set import end
lnName = 4; % line number of protein name
lnPDB = 71; % line number of PDB entry
lnSSstart = 63;
lnSSend = 70;
GenerateSELCON = 1; % if 1 creates a SELCON input file

%% Obtain the list of files
listing = dir(inputPath); % Lists the contents of the working directory
fileNames = {listing.name}; %
inputSpecCD = '%f %f %f %f %f %f'; % Read in the CD data, ie. WL, CD, HT, Abs, etc...

dataStart = SSWLstart; % Search for the start
dataEnd = SSWLend; % Search for the end - not really necessary but handy to have

%% Process each file
for i = 3 : (length(listing))
   inputFilename = fileNames(i);
   inputFilename = cell2mat(inputFilename);
   inputFile = char(strcat('.',filesep,inputPath,filesep,inputFilename));
   fh = fopen(inputFile,'rt'); 
   
   % Find the beginning
   lnStartCD = 0; % CD spectrum starting line
   while ~feof(fh)
       tline = fgetl(fh);
       lnStartCD = lnStartCD + 1;
       if ischar(tline) && ~isempty(strfind(tline, dataStart))
           break;
       end
   end

   % Find the data end
   lnEndCD = lnStartCD;
   while ~feof(fh)
       tline = fgetl(fh);
       lnEndCD = lnEndCD + 1;
       if ischar(tline) && ~isempty(strfind(tline, dataEnd))
           break;
       end
   end
      
   % Rewind the file handle and read CD data between lnStartCD and lnEndCD
   frewind(fh);
   lineNum = 0;
   r = 1;
   %CDdata = zeros(66,1);
   while ~feof(fh)
      currentLine = fgetl(fh);
      lineNum = lineNum + 1;
      while lineNum >= (lnStartCD-1) && lineNum <= lnEndCD
         currentLine = fgetl(fh);
         CDdataRaw = cell2mat(textscan(currentLine,inputSpecCD));
         CDdata(r,1) = CDdataRaw(2);
         lineNum = lineNum + 1;
         r = r + 1;
         if lineNum == lnEndCD
            break;
         end
      end
   end
   
   % SS = zeros(8,1);
   filetext = fileread(inputFile);
   proteinName = regexp(filetext, '(?<=Protein Name[\ ]*)[\w]+', 'match');
   proteinName = cell2mat(proteinName(1,1));
   PDBID = cell2mat(regexp(filetext, '(?<=PDB ID[\ ]*)[0-9]?[0-9,a-z]{3}', 'match'));
   SS(1,1) = regexp(filetext, '(?<=DSSP value: alpha helix[\ ]*)[0]\.?[0-9]{3}', 'match');
   SS(2,1) = regexp(filetext, '(?<=DSSP value: 3-10 helix[\ ]*)[0]\.?[0-9]{3}', 'match');
   SS(3,1) = regexp(filetext, '(?<=DSSP value: pi helix[\ ]*)[0]\.?[0-9]{3}', 'match');
   SS(4,1) = regexp(filetext, '(?<=DSSP value: beta strand[\ ]*)[0]\.?[0-9]{3}', 'match');
   SS(5,1) = regexp(filetext, '(?<=DSSP value: beta bridge[\ ]*)[0]\.?[0-9]{3}', 'match');
   SS(6,1) = regexp(filetext, '(?<=DSSP value: bonded turn[\ ]*)[0]\.?[0-9]{3}', 'match');
   SS(7,1) = regexp(filetext, '(?<=DSSP value: bend[\ ]*)[0]\.?[0-9]{3}', 'match');
   SS(8,1) = regexp(filetext, '(?<=DSSP value: loop or irregular[\ ]*)[0]\.?[0-9]{3}', 'match');
   %SS = cell2mat(SS);

   CDdataLen = size(CDdata,1);
   j = i-2; % use the i counter
   CDdataFinal(1,j) = {proteinName};
   CDdataFinal(2,j) = {PDBID};
   
   for k = 3 : CDdataLen+2
      CDdataFinal(k,j) = num2cell(CDdata(k-2,1));
   end
   
   for l = 1 : length(SS)
      CDdataFinal(l+CDdataLen+2,j) = num2cell(SS(l,1));
   end
 
   if GenerateSELCON == 1
      CDstr = CDdata'; % make a copy to output to SOMSpec
      CDstr = mat2str(CDstr); % Convert to a string
      CDstr = CDstr(2:end-1); % Strip the brackets
      WLstart = SSWLstart;
      WLend = SSWLend;
      WLstep = 1;
      
      sampleName = CDdataFinal(2,j);
      sampleName = cell2mat(sampleName);
      SSJSelProc(WLstart,WLend,WLstep,CDstr,sampleName);
   end
      
end
   
   % Write the file
   CDdataFinal = cell2table(CDdataFinal);
   writetable(CDdataFinal,'SP175txt');

fclose('all');
fprintf('Done.');
end