%
% Copyright (C) Dale L. Ang
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Reads in the SOM files and plots them, similar to the real-time plots option. Can be run here or by the GUI.
%

function [Parameters] = SSSOMPLOT(Filenames,Parameters)
% Specify style, SOM location and node to plot
SOMplotStyle = 3; % 1 = general overview, 2 = individual properties and vector sums, 3 = node plot
SOMlocation = '/home/dale/dev/SOMSpec/TESTING/SOM-10-2000';
nodeX = 1;
nodeY = 1;

% Set to 0 if running as script
%Parameters.isGUI = 0;

if Parameters.isGUI == 0 % Set up structs
   savePath = SOMlocation;

   Filenames = struct(     'savePath',          savePath);
   
   Parameters = struct(    'mapLength',         1, ...
                           'numIterations',     1,...
                           'numStructures',     1, ...
                           'numLines',          1, ...
                           'numCols',           1, ...
                           'L0',                1, ...
                           'numBMUS',           1, ...
                           'modelSize',         1, ...
                           'modelSizeInt',      1, ...
                           'testSize',          1, ...
                           't1',                1, ...
                           'trainingSetSize',   1, ...
                           'radius0',           1, ...
                           'refSize',           1, ...
                           'validate',          1, ...
                           'wavelengthLong',    1, ...
                           'wavelengthShort',   1, ...
                           'waveUnits',         '', ...
                           'nodeXtoPlot',       nodeX, ...
                           'nodeYtoPlot',       nodeY, ...
                           'SOMplotStyle',      SOMplotStyle, ...
                           'warning',           0);
end

%% Define input variables here. All other options come from the training set files.
ext = '.txt';
%refPath = strcat(savePath,filesep,refPathCM);
%structureFiles = strcat(refPath,filesep,'structures');
%weightFiles = strcat(refPath,filesep,'weightMap');
%parameterFile = load(strcat(refPath,filesep,'parameters.txt'));
Filenames.refPathSOM = Filenames.savePath;
Filenames.testSetFileExt = '.txt';
sumNodes = zeros(Parameters.mapLength,Parameters.mapLength);

%% Read in the parameters of the SOM
[Parameters] = readParameters(Filenames,Parameters);

if Parameters.nodeXtoPlot > Parameters.mapLength || Parameters.nodeYtoPlot > Parameters.mapLength
   fprintf('Specify a node within the map.\n');
   Parameters.warning = 1;
   % Parameters. % return an error to the GUI
   return
end

%% Load the weightMap and structures files.
weightMap = zeros(Parameters.mapLength,Parameters.mapLength,Parameters.modelSize);
structures = zeros(Parameters.mapLength,Parameters.mapLength,Parameters.numStructures+1);

for i = 1:Parameters.modelSizeInt
   nameW = 'weightMap';
   numberW = num2str(i);
   weightMap(:,:,i) = load([Filenames.savePath,filesep,nameW,numberW,ext]);
end

for i = 1:(Parameters.numStructures+1)
   nameW = 'structures';
   numberW = num2str(i);
   structures(:,:,i) = load([Filenames.savePath,filesep,nameW,numberW,ext]);
end

for x = 1:Parameters.mapLength
   for y = 1:Parameters.mapLength
      sumCurrentNode = sum(weightMap(x,y,:)); % Getting it into the right format to plot
      sumNodes(x,y) = sumCurrentNode; % update the sumNode map for plotting
   end 
end

structuresNode = structures(Parameters.nodeXtoPlot,Parameters.nodeYtoPlot,:);
structuresNodeSq = squeeze(structuresNode)';
parametersLegend = zeros(1,length(structuresNodeSq)-1);

for i = 1:length(parametersLegend)
   format bank;
   parametersLegend = strcat(parametersLegend,num2str(round(structuresNodeSq(i),2)),',');
end

parametersLegend = parametersLegend(1:end-1); % Crop the last comma off. Quick adn dirty.

%% Generate final plots
if Parameters.SOMplotStyle == 1
   figure(1);

   subplot(1,3,1);
   surf(weightMap(:,:,Parameters.modelSizeInt/2));
   axis ([0 Parameters.mapLength 0 Parameters.mapLength -10 10]);
   title('SOM at midpoint');

   subplot(1,3,2);
   surf(sumNodes(:,:));
   axis on;
   title('Vector sum');

   subplot(1,3,3);
   xslice = [Parameters.mapLength/2 Parameters.mapLength];
   yslice = [Parameters.mapLength/2 Parameters.mapLength];
   zslice = [1 Parameters.modelSize/2];
   slice(weightMap(:,:,:),xslice,yslice,zslice);
   axis ([0 Parameters.mapLength 0 Parameters.mapLength 0 Parameters.modelSizeInt]);
   title('Volume');

elseif Parameters.SOMplotStyle == 2
   figure(2);
   % SOM and SS plots
   midPoint = round(Parameters.modelSizeInt/2); % Values of weightMap to plot. Analogous to one wavelength of each spectrum.
   %midPoint = round(midPoint);
   figure(2); % Generate figure window
   axis on;
   subplot(2,4,1);
   surf(weightMap(:,:,midPoint));
   title('Plot of spectra map at midpoint.');

   subplot(2,4,2);
   surf(structures(:,:,1));
   title('Property 1');

   subplot(2,4,3);
   surf(structures(:,:,2));
   title('Property 2');

   subplot(2,4,4);
   surf(structures(:,:,3));
   title('Property 3');

   subplot(2,4,5);
   surf(structures(:,:,4));
   title('Property 4');

   subplot(2,4,6);
   surf(structures(:,:,5));
   title('Property 5');

   subplot(2,4,7);
   surf(structures(:,:,6));
   title('Property 6 - BMUs');

   subplot(2,4,8);
   surf(sumNodes(:,:));
   axis on;
   title('Vector sum');

elseif Parameters.SOMplotStyle == 3
   figure(3);
   hold on;
   %xlim([1 Parameters.modelSizeInt]);
   %set(gca,'xdir','reverse')

   % Set x axis tick labels
   ax = gca;
   ax.XLim = [1 Parameters.modelSizeInt];
   ax.XDir = 'reverse';
   %ax.XTickMode = 'manual';
   %ax.XTick = [Parameters.wavelengthShort Parameters.wavelengthLong];
   %ax.XTickLabelMode = 'manual';

   spectrumWeight = weightMap(Parameters.nodeXtoPlot,Parameters.nodeYtoPlot,:);
   spectrumWeightSqueezed = squeeze(spectrumWeight);
   
%    spectrumStructure = structures(Parameters.nodeXtoPlot,Parameters.nodeYtoPlot,:);
%    spectrumStructureSqueezed = squeeze(spectrumStructure);
%    spectStructRnd = round(spectrumStructureSqueezed,2);
   
   subplot(1,1,1);
   
   plot(spectrumWeightSqueezed,'DisplayName',strcat('X',num2str(Parameters.nodeXtoPlot),',Y',num2str(Parameters.nodeYtoPlot),', P:',parametersLegend));
   title(strcat('Spectrum of nodes'));
   %legend(strcat(num2str(Parameters.nodeXtoPlot),',',num2str(Parameters.nodeYtoPlot)));   
   
   legend;
end

Parameters.warning = 0; % Reset the warning bit

end

