%
% Copyright (C) Dale L. Ang 20181017
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% This function searches the reference set map for nodes matching the input data.
% The properties of the best matching units (BMUs) are then applied in a
% distance dependent manner.
%
%  20181017 - Final v1.2
%

function  SSTEStruct(Filenames,Parameters,weightMap,structures,coordsMatrix)
   testSetFile = strcat(Filenames.savePath,Filenames.testSetFileVa,Filenames.testSetFileExt);
   testSet = load(testSetFile);
   %bestNRMSD = zeros(length(Parameters.concError),2);
   %bestConc = 100000;
   
   % Resample the sampleVector to modelSizeInt
   sampleVector = testSet(1:Parameters.modelSize,Parameters.currentSpectrum)';
   [sampleVectorInt] = interpolateSpectrum(Parameters,sampleVector);
   inputVectorInt = sampleVectorInt; 

   %% Loop over the conc range to find the optimal NRMSD - TODO
   if Parameters.spectraIsIR == 1 % Check if scaling is required, else skip.
      bestConc = 1.0;
   else
      [bestConc] = findOptimumScalingFactor(Parameters,weightMap,inputVectorInt); % Call function to test for best NRMSD
      %[~,NRMSDindex] = min(bestNRMSD);
      %bestConc = bestNRMSD(NRMSDindex(2));
   end
   
   %% After finding the optimal concScaling/NRMSD values optimisation
   % do the real testing here with the optimal concError value
  
   % Apply the concenctration correction factor
   inputVectorIntConc = inputVectorInt.*bestConc; %concError(bestConc);
   
   %% Search for BMUs of the test spectrum.
   % Initialise the various matrices
   weightVector = zeros(1,Parameters.modelSizeInt);
   contrib = zeros(Parameters.numStructures,Parameters.numBMUS);
   outputs = zeros(Parameters.numStructures,Parameters.numBMUS);
	outputsum = zeros(1,Parameters.numStructures);
   outputvector = zeros(1,Parameters.numStructures);
   normalisedSpectrum = zeros(1,Parameters.modelSizeInt);
   BestMU = zeros(Parameters.numBMUS,Parameters.modelSizeInt); % Contains SOM spectrum for each BMU.
   BMUSpectralModel = zeros(Parameters.numBMUS,Parameters.modelSizeInt);
   bestX = zeros(Parameters.numBMUS,1);
   bestY = zeros(Parameters.numBMUS,1);
   bestDistance = zeros(Parameters.numBMUS,1);
   bestDistance(:,1) = 10000*Parameters.mapLength; % initialise to a very large distance
   
   % Find all the BMUs by iterating over the map and finding the closest spectral matches
   for bmu = 1:Parameters.numBMUS
      for x = 1:Parameters.mapLength
         for y = 1:Parameters.mapLength
            weightVector(1,:) = weightMap(x,y,:); % vectorised
            d = sqrt(sum((inputVectorIntConc - weightVector).^2)); % Calculate distance from this node to BMU.
            % For the first BMU, find the best/smallest value of d and save the d, X and Y values 
            if bmu == 1 && d < bestDistance(bmu,1)
               bestDistance(bmu) = d;
               bestX(bmu,1) = x;
               bestY(bmu,1) = y;
               % For the rest of the BMUs,    
            elseif (bmu > 1 && d <= bestDistance(bmu,1) && d > bestDistance(bmu-1,1)) % Rank BMUs in order of distance.
               bestDistance(bmu) = d;  
               bestX(bmu,1) = x;
               bestY(bmu,1) = y;
            else
               continue;
            end
         end
      end
      % Check for invalid bmu coordinates
      if bestX(bmu,1) < 1
         bestX(bmu,1) = bestX(bmu-1,1);
      end
      if bestY(bmu,1) < 1
         bestY(bmu,1) = bestY(bmu-1,1);
      end
   end

   % Node parameters are assigned using the distance of the poorest match - see AlgoC2/3
   BMUradius = bestDistance(end); % Set the BMUradius to the largest distance
   
   %% Calculate contributions from each BMU - algorithm should match training algorithm
   p = zeros(Parameters.numBMUS,1);
   
   if bestDistance(bmu,1) ~= 0
      for bmu = 1:Parameters.numBMUS
         %p(bmu,1) = exp(-bestDistance(bmu,1)*0.05); % AlgoA - Smoother than 1/distance
         %p(bmu,1) = 1/bestDistance(bmu,1); % AlgoB - produces harsh map peaks
         %p(bmu,1) = exp(-(bestDistance(bmu,1)^2)/(2*Parameters.radius0^2)); % AlgoC - traditional Gaussian
         %p(bmu,1) = exp(-(bestDistance(bmu,1)^2)/(2*BMUradius^2)); % AlgoC - traditional Gaussian
         %p(bmu,1) = exp(-(bestDistance(bmu,1)^2)/(2*BMUradius^2)); % AlgoC2 - Gaussian based on BMUradius
         p(bmu,1) = exp(-(bestDistance(bmu,1)^2)/(BMUradius^2)); % AlgoC3 - Gaussian based on BMUradius
         %p(bmu,1) = exp(-(bestDistance(bmu,1)^2)/(Parameters.radius0^2)); % AlgoD - tighter Gaussian
         %localRadius = max(bestDistance);
         %p(bmu,1) = exp(-(bestDistance(bmu,1)^2)/(2*localRadius^2)); % AlgoG - Gaussian based on radius of BMUs, not map
         %p(bmu,1) = 1/(bestDistance(bmu,1)^2); % AlgoE - Inverse square - not very good
      end
   end

   if sum(bestX) > Parameters.numBMUS && sum(bestY) >= Parameters.numBMUS % This was added as a check if BMU locating failed
      %% Read in the structures of each BMU from the SOM
      for sn = 1:Parameters.numStructures
         for bmu = 1:Parameters.numBMUS
            contrib(sn,bmu) = structures(bestX(bmu),bestY(bmu),sn); % Obtain properties of BMUs.
            outputs(sn,bmu) = (p(bmu).*contrib(sn,bmu)); % Apply BMU contributions to the property values.
         end
      end

      %% Add the contributions.
      % Sum over BMUs
      for sn = 1:Parameters.numStructures
         outputsum(1,sn) = sum(outputs(sn,1:Parameters.numBMUS));
      end

      %% Normalise properties, assignments should sum to 1.
      summ = sum(outputsum);
      outputvector = outputsum/summ;
       
      %% Read in each BMU's spectrum and save along with x and y coords
      % and generate spectrum from BMUs to compare with the input spectrum.
      for t = 1:Parameters.numBMUS
            %bestMU(t,:,:,:) = weightMap(bestX(t),bestY(t),1:Parameters.modelSizeInt); % Store all BMU details
            BestMU(t,:) = weightMap(bestX(t),bestY(t),1:Parameters.modelSizeInt); % Populate with each SOM BMU spectrum
      end

      % Select spectrum from first BMU for outputting
      BMUAver = BestMU(1,1:Parameters.modelSizeInt);
      
      rangeBMUAver = (max(BMUAver)-min(BMUAver));
      if rangeBMUAver == 0
         rangeBMUAver = 1;
      end

      %% Calculate the NRMSD and RMSD
      [spectNRMSD,spectRMSD] = calculateNRMSDandRMSD(BMUAver,inputVectorIntConc,rangeBMUAver);
      
      %% Output everything to files - appending only
      spectRMSDplusNumbers = [Parameters.currentSpectrum,spectRMSD];
      dlmwrite(strcat(Filenames.savePathTest,'Spectra_RMSD',Filenames.testSetFileExt),spectRMSDplusNumbers,'-append','precision',9);

      allNRMSDs = [Parameters.currentSpectrum,bestConc,spectNRMSD];
      dlmwrite(strcat(Filenames.savePathTest,'Spectra_NRMSD',Filenames.testSetFileExt),allNRMSDs,'-append','precision',9);
      
      structuresToFile = outputvector(:);
      
      structuresToFile1 = structuresToFile';
      structuresPlusNumber = [Parameters.currentSpectrum,structuresToFile1];
      residuals = BMUAver-inputVectorIntConc;
      residualsWithNumbers = [Parameters.currentSpectrum,residuals];

      dlmwrite(strcat(Filenames.savePathTest,'Spectra_Residuals',Filenames.testSetFileExt),residualsWithNumbers,'-append','precision',9);
      dlmwrite(strcat(Filenames.savePathTest,'Spectra_Real',Filenames.testSetFileExt), inputVectorIntConc, '-append','precision',9);
      dlmwrite(strcat(Filenames.savePathTest,'Spectra_Predicted',Filenames.testSetFileExt), BMUAver, '-append','precision',9);
      dlmwrite(strcat(Filenames.savePathTest,'Secondary_structure_predictions',Filenames.testSetFileExt),structuresPlusNumber,'-append','precision',4); % Finally... output the predicted structures

      %% Generate output plots
      generateTestPlots(Parameters,Filenames,bestX,bestY,coordsMatrix,inputVectorIntConc,BMUAver,residuals,spectNRMSD);
      
   end
fclose('all');    
end