%
% Copyright (C) Dale L. Ang
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Generates the SOM from the input training set found in ./ 
% Calls SSTRWeight to train the map, which in turn calls SSTRStruct to
% assign the structure parameters.
%
% 20181017 - Final v1.2
%

function [Filenames,Parameters] = SSTRAIN(Filenames,Parameters)

   refSetFile = strcat(Filenames.refSetFileVa,Filenames.refSetFileExt); % Reference set file name. @@@VAL
   Filenames.refPath = strcat(Filenames.savePath,refSetFile); % Location of input file.

   if Parameters.validate == 1
      Filenames.savePathSOM = strcat(Filenames.savePath,Filenames.SOMprefix,'-',Parameters.W);
   else
      Filenames.savePathSOM = strcat(Filenames.savePath,Filenames.SOMprefix);
   end

   %% Call file read function to calculate number of lines and columns
   file = Filenames.refPath; % Set the location of the file for the get LinesColumsn function
   [numLines,numCols] = getLinesColumns(file);
   Parameters.numLines = numLines;
   Parameters.numCols = numCols;

   %% Calculate size of training fileset (weightSize)
   Parameters.modelSize = Parameters.numLines - Parameters.numStructures; % Remove the last 6 (or however many) lines from the row count
   if Parameters.modelSize < 0
      error('Invalid training set. Please specify.'); % Checks that the correct delimiter has been defined.
   end

   %% Calculate remaining variables
   %Parameters.refSize = Parameters.numCols; % Number of columns
   Parameters.trainingSetSize = Parameters.numCols;
   if Parameters.trainingSetSize < 2
      error('Training set too small. You need more than 1 entry for training.'); % Checks that the training set is large enough.
   end
   
   %% Make directory for output
   if exist(Filenames.savePathSOM,'dir') ~= 7
      mkdir(Filenames.savePathSOM); % TODO rename folder to new name and show warning in GUI
   else
      fprintf('WARNING: Output folder already exists!\nResults will replace existing files.\n');
      if Parameters.validate == 0
         pause; % maybe be better to quit? except... quit closes MATLAB
      end
   end

   % Calculate number of points for interpolation
   [Parameters] = calculateInterpolSteps(Parameters);
   
   %% Trains the map with SSTRWeight then assigns structures with SSTRStruct.
   [Filenames,Parameters] = SSTRWeight(Filenames,Parameters);
   if Parameters.isCancelled == 1
      return;
   end
   
   %% Write parameters file
   parameters = {Parameters.mapLength,Parameters.radius0,Parameters.numIterations,Parameters.L0,Parameters.numStructures,Parameters.numBMUS,Parameters.modelSizeInt,Parameters.trainingSetSize,Parameters.t1,Parameters.wavelengthLong,Parameters.wavelengthShort,Parameters.waveUnits};
   parametersVarNames = {'mapLength','radius0','numIterations','L0','numStructures','numBMUS','modelSizeInt','trainingSetSize','t1','wavelengthLong','wavelengthShort','waveUnits'};
   parametersOut = array2table(parameters,'VariableNames',parametersVarNames);
   
   %dlmwrite(strcat(Filenames.savePathSOM,filesep,'parameters',Filenames.refSetFileExt),parametersOut,'-append');
   writetable(parametersOut,strcat(Filenames.savePathSOM,filesep,'parameters',Filenames.refSetFileExt));
   
   %% Finished
   fprintf('Training complete.\n');
   fclose('all');
   %clear; % Can't pass variables back if they're cleared!
end