%
% Copyright (C) Dale L. Ang
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Constructs the structures map of the SOM for structure estimation.
% Called by SSTRWeight.m.
%
%  Called by SSTRWeight
%

function [structures] = SSTRStruct(Filenames,Parameters,structures,weightMap)
   
   % Initialise things
   coordsMatrix = zeros(Parameters.trainingSetSize,3); % Initialising coordsMatrix for storing BMU locations on the map.
   refSet = load(Filenames.refPath); % load the reference set.
   inputMatrix = refSet'; % Transpose the refSet.
   wm = zeros(1,Parameters.modelSizeInt); % initialising weight map.
   contrib = zeros(Parameters.numStructures,Parameters.numBMUS); % Stores the SS contribution
   outputs = zeros(Parameters.numStructures,Parameters.numBMUS);
   outputsum = zeros(Parameters.numStructures,1);
   %NRMSD = zeros(Parameters.trainingSetSize,2); % initialising.

   %% Iterate over the input and find the best matching nodes to construct the coordsMatrix list
   for protein = 1:Parameters.trainingSetSize
      
      % Resample the inputVector to modelSizeInt
      inputVector = inputMatrix(protein,1:Parameters.modelSize);
      [sampleVectorInt] = interpolateSpectrum(Parameters,inputVector);
      
      %Initialise the rest - initial bmuDistance must be very large to avoid crashes
      bmuDistance = 1000000*Parameters.mapLength; % Initial arbitrary size @@@FIXME
      weightVectorInt = zeros(1,Parameters.modelSizeInt);
      BMU_x = -1; % -1 value will cause crash if it doesn't get updated (rare occurance). @@@FIXME?
      BMU_y = -1; % But not the cause of the occasional problem
      
      % Find the BMU to apply the structure to
      [BMU_x,BMU_y] = locateBMUs(Parameters,weightMap,sampleVectorInt,bmuDistance,weightVectorInt,BMU_x,BMU_y);

      % Read in the structure information of the current training set spectrum
      proteinStructure = inputMatrix(protein,Parameters.modelSize+1:Parameters.modelSize+Parameters.numStructures); % Read in structure contents of input spectrum
      
      %LOOV failing here - BMU_x/y not being returned by locateBMU when doing LOOV.
      structures(BMU_x, BMU_y, 1:Parameters.numStructures) = proteinStructure; % populate with the structure values from the input spectrum
      
      % Set up the output file for locating the matching node for each input structure from the training set
      coordsMatrix(protein,1) = protein;
      coordsMatrix(protein,2) = BMU_x;
      coordsMatrix(protein,3) = BMU_y;
      
      % Calculate NRMSD between the test set spectrum and the identified best matching node spectrum and set the BMU bit
      wm(:) = weightMap(BMU_x,BMU_y,1:Parameters.modelSizeInt); % weightMap at these coordinates
      structures(BMU_x,BMU_y,Parameters.numStructures+1) = 1; % Set this node as a BMU
      
   end
  
   %% Now iterate over the map to complete SS assignment.
   for x = 1:Parameters.mapLength
      for y = 1:Parameters.mapLength
         if structures(x,y,Parameters.numStructures+1) == 0 % Is it a BMU or just a plain node? If BMU ignore.
            bestX(1:Parameters.numBMUS,1) = -1; % Keep as -1 to easily find issues @@@FIXME
            bestY(1:Parameters.numBMUS,1) = -1; % and here too
            bestDistance(1:Parameters.numBMUS,1) = 1000000*Parameters.mapLength; % Initialise at a very large distance @@@FIXME
            for bmu = 1:Parameters.numBMUS
               for entry = 1:Parameters.trainingSetSize
                  bmuX = coordsMatrix(entry,2); % The coordinates of the current BMU
                  bmuY = coordsMatrix(entry,3);
                  distance = sqrt(((bmuX - x)^2) + ((bmuY - y)^2)); % Find closest BMU to current node
                  if bmu == 1 && distance < bestDistance(bmu,1) 
                     bestDistance(bmu,1) = distance;
                     bestX(bmu,1) = bmuX;
                     bestY(bmu,1) = bmuY;
                  elseif (bmu > 1 && distance <= bestDistance(bmu,1) && distance > bestDistance(bmu-1,1)) % I.e. must be better than previous version of self, but worse than previous BMU.
                     bestDistance(bmu,1) = distance;
                     bestX(bmu,1) = bmuX;
                     bestY(bmu,1) = bmuY;
                  else
                     continue; % If the distance is not smaller, ignore and go to next BMU in list
                  end
               end
            end
            
            % Node parameters are assigned using the distance of the poorest match - see AlgoC2/3
            BMUradius = bestDistance(end); % Set the BMUradius to the most distant BMU

            %% Contribution of the best node from BMU
            p = zeros(Parameters.numBMUS,1);

            if bestDistance(bmu,1) ~= 0
               for bmu = 1:Parameters.numBMUS % Calculating the proportion this node contributes. Alternative algorithms.
                  %p(bmu,1) = exp(-bestDistance(bmu,1)*0.05); % AlgoA - Smoother than 1/distance
                  %p(bmu,1) = 1/bestDistance(bmu,1); % AlgoB - produces harsh map peaks
                  %p(bmu,1) = exp(-(bestDistance(bmu,1)^2)/(2*Parameters.radius0^2)); % AlgoC - traditional Gaussian
                  %p(bmu,1) = exp(-(bestDistance(bmu,1)^2)/(2*BMUradius^2)); % AlgoC2 - Gaussian based on BMUradius
                  p(bmu,1) = exp(-(bestDistance(bmu,1)^2)/(BMUradius^2)); % AlgoC3 - Gaussian based on BMUradius
                  %p(bmu,1) = exp(-(bestDistance(bmu,1)^2)/(Parameters.radius0^2)); % AlgoD - tighter Gaussian
                  %localRadius = max(bestDistance);
                  %p(bmu,1) = exp(-(bestDistance(bmu,1)^2)/(2*localRadius^2)); % AlgoG - Gaussian based on radius of BMUs, not map
                  %p(bmu,1) = 1/(bestDistance(bmu,1)^2); % AlgoE - Inverse square - not very good
               end
            elseif bestDistance(bmu,1) == 0 % to catch any cases where the node is itself and divide by zero occurs
               continue;
            end
            
            % Iterate through the structures, calculating the contributions.
            for sn = 1:Parameters.numStructures
               for bmu = 1:Parameters.numBMUS
                  contrib(sn,bmu) = structures(bestX(bmu,1),bestY(bmu,1),sn);
               end
            end

            for sn = 1:Parameters.numStructures
               for bmu = 1:Parameters.numBMUS
                  outputs(sn,bmu) = (p(bmu,1).*contrib(sn,bmu)); % Node SS becomes distance modifier multiplied by the BMU SS
               end
            end

            for sn = 1:Parameters.numStructures
               outputsum(sn) = sum(outputs(sn,1:Parameters.numBMUS));
            end

            % Normalise, SS assignments should sum to 1.
            summ = sum(outputsum);
            outputVector = outputsum/summ;
            
            % Update the structures map.
            for sn = 1:Parameters.numStructures
               structures(x,y,sn) = outputVector(sn);
            end
         end
      end
   end

   %% Save the structuresX files.
   sizeStructures3 = size(structures,3);
   path = strcat(Filenames.savePathSOM,filesep);
   name = 'structures';
   ext = Filenames.refSetFileExt;
   for i = 1:sizeStructures3
      number = num2str(i);
      dlmwrite([path,name,number,ext],structures(:,:,i),'precision',9);
   end

   %% Save the coordsMatrix
   coord = 'coordsMatrix';
   dlmwrite(strcat(path,coord,ext),coordsMatrix);
end
