%
% Copyright (C) Dale L. Ang
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Trains the SOM by randomly selecting from the the input training set.
% Subsequently calls SSTRStruct.m to assign properties to each node.
%
%  20181017 - Final v1.2
%

function [Filenames,Parameters] = SSTRWeight(Filenames,Parameters)
   structures = zeros(Parameters.mapLength,Parameters.mapLength,Parameters.numStructures+1); % structures matrix initialised+1 to include training set locations. 
   weightMap = rand(Parameters.mapLength,Parameters.mapLength,Parameters.modelSizeInt); % Generate the random map initialised with random values between 0-1.
   refSet = load(Filenames.refPath); % Load the refSet

   radius = Parameters.radius0; % Set the neighbourhood radius to the initial value
   learningRate = Parameters.L0; % Set the learningRate to the initial value
   refSetTransp = refSet'; % transpose matrix
   
   if Parameters.realtimePlots == 1 || Parameters.displayFinalPlots == 1
      radiusPlot = zeros(1,Parameters.numIterations); % Initialising neighbourhood radius vector (for plotting).
      learningRatePlot = zeros(1,Parameters.numIterations); % Initialising learning rate vector (for plotting).
      sumNodes = zeros(Parameters.mapLength,Parameters.mapLength); % Required for plotting
      figure(1); % Generate figure window for live surface plot
   end

   % Create progress bar
   if Parameters.displayProgressBar == 1
      % Size and position UI
      displayDimensions = get(0,'ScreenSize');
      xRez = displayDimensions(3);
      yRez = displayDimensions(4);
      UIwidth = 500; % Set the GUI width here
      UIheight = 250; % Set the GUI height here
      UIxpad = (xRez-UIwidth)/2;
      UIypad = (yRez-UIheight)/2;
      
      f = uifigure;
      f.Position = [UIxpad UIypad UIwidth UIheight];
      d = uiprogressdlg(f,'Title','Please Wait',...
      'Message','Training the SOM','Cancelable','on','CancelText','Cancel training.');
      
      %close(d);
   end
   
   %% Construct the map
   for iteration = 1:Parameters.numIterations
      if mod(iteration,100) == 0 % Update progress every 1000 iterations
         disp(iteration);
      end
      
      % Update the progress bar - alternate version for LOOV
      if Parameters.displayProgressBar == 1 && Parameters.validate == 0
         progress = round(iteration/Parameters.numIterations,2);
         d.Value = progress;
         if d.CancelRequested == 1
            Parameters.isCancelled = 1;
            close(f);
            return;
         end
      % Show alternate bar for LOOV progress
      elseif Parameters.validate == 1
         if Parameters.displayProgressBar == 1
            progress = round(Parameters.V/Parameters.trainingSetSize,2);
            d.Value = progress;
            if d.CancelRequested == 1
               Parameters.isCancelled = 1;
               close(f);
               return;
            end
         end  
      end
      
      % Initialise everything
      bmuDistance = 10*Parameters.mapLength; % Just an initial size - it is quickly made equal to distance1
      randNumb = randi([1,Parameters.trainingSetSize],1); % Obtain a 1x1 random number from 1 to trainingSetSize.
      weightVectorInt = zeros(1,Parameters.modelSizeInt);
      currentNode = zeros(1,Parameters.modelSizeInt); 
      BMU_x = -1;
      BMU_y = -1;
      
      % Initialise and populate the sampleVector, ignoring the SS values
      sampleVector = zeros(1,Parameters.modelSize); % initialise the input vector.
      for mn = 1:Parameters.modelSize % mn is modelnumber.
         sampleVector(mn) = refSetTransp(randNumb,mn); % This is where the spectrum is imported, ignoring the SS values.
      end

      % Resample the inputVector to modelSizeInt
      [sampleVectorInt] = interpolateSpectrum(Parameters,sampleVector);
      
      %% Display the resampled inputs in real time during training
      if Parameters.displayResampledSpectra == 1 && Parameters.realtimePlots == 0 % Makes sure both options aren't enabled
         p1 = subplot(2,1,1);
         plot(sampleVector);
         axis(p1,[0 Parameters.modelSize -20 20]);
         p2 = subplot(2,1,2);
         plot(sampleVectorInt);
         axis(p2,[0 Parameters.modelSizeInt -20 20]);
         pause(0.02);
      end
      
      %% Find the BMU
      [BMU_x,BMU_y] = locateBMUs(Parameters,weightMap,sampleVectorInt,bmuDistance,weightVectorInt,BMU_x,BMU_y);

      %% Update the current node, if distance < current radius make a weighted change
      for x = 1:Parameters.mapLength % Iterate over map updating the newWeights to match inputs.
         for y = 1:Parameters.mapLength
            currentNode(:) = weightMap(x,y,1:Parameters.modelSizeInt); % Set current as focus
            distance = sqrt(((BMU_x - x)^2) + ((BMU_y - y)^2)); % distance of currentNode from BMU
            if (distance <= radius) % Update the current node
               thetaDist = exp(-(distance^2)/(2*radius^2));
               vectorDiff = (sampleVectorInt - currentNode);
               weightMap(x,y,:) = currentNode + learningRate*thetaDist*vectorDiff; % update the node with the new weighting
            end
         end
      end
      
      %% Live ploting section
      if Parameters.realtimePlots == 1
         plotRealtime(Parameters,refSet,radiusPlot,learningRatePlot,sumNodes,weightMap); % Do the realtime plotting
      end

      % Update the radius of the neighbourhood of the BMU and the learningRate
      radiusPlot(1,iteration) = radius; % Update it for plotting
      radius = (Parameters.radius0)*exp(-iteration/Parameters.t1); % Exponentially decay the neighbourhood radius.
      learningRatePlot(1,iteration) = learningRate; % Update it for plotting
      learningRate = Parameters.L0*exp(-iteration/Parameters.t1); % Exponentially decay the learning rate.
   end

   % Close progress bar
   if Parameters.displayProgressBar == 1
      close(d);
      close(f);
   end
   
   if Parameters.realtimePlots == 1
      pause; % Pause to admire the beautiful plots.
   end
   
   %% Assign secondary structure values
   [structures] = SSTRStruct(Filenames,Parameters,structures,weightMap); % Call the structure assigning function

   %% Save the weightMap to a bunch of files.
   sizeWeightMap3 = size(weightMap,3);
   path = strcat(Filenames.savePathSOM,filesep);
   name = 'weightMap';
   ext = Filenames.refSetFileExt;
   
   for i = 1:sizeWeightMap3
      number = num2str(i);
      dlmwrite([path,name,number,ext],weightMap(:,:,i),'precision',9);
   end
   
   %% Generate final plots
   if Parameters.displayFinalPlots == 1 % Display final trained map plots
      plotFinalTrained(Parameters,sumNodes,weightMap,structures); % Do the realtime plotting
   end
end